@@ -1,46 +1,26 @@
 /* 
- * Copyright (C) 2005 Christian Persch
+ * Copyright © 2005 Christian Persch
+ * Copyright © 2005 Robert O'Callahan
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your option)
+ *  any later version.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * The Original Code is GNOME Web Photo code.
- *
- * The Initial Developer of the Original Code is
- * Christian Persch <chpe@gnome.org>.
- * Portions created by the Initial Developer are Copyright (C) 2004, 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
  * * some parts of code copied from mozilla/layout/base/nsDocumentViewer.cpp:
  * *  Dan Rosen <dr@netscape.com>
  * *  Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
  *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK *****
- *
- * $Id: Writer.cpp,v 1.13 2005/11/26 16:14:56 chpe Exp $
+ * $Id: Writer.cpp 85 2007-01-29 18:51:07Z chpe $
  */
 
 #include "mozilla-config.h"
@@ -245,7 +225,7 @@ Writer::Write()
     }
 
     retval = Finish();
-    status = status = retval ? "OK" : status[0] != '\0' ? status : "FAILED";
+    status = retval ? "OK" : status[0] != '\0' ? status : "FAILED";
     LOG ("\n");
   }
 
@@ -410,14 +390,12 @@ PNGWriter::WriteSurface(nsIDrawingSurface *aSurface,
       for (PRUint32 j = 0; j < aWidth; ++j)
         {
           /* v is the pixel value */
-#if defined(IS_BIG_ENDIAN)
+#ifdef WORDS_BIGENDIAN
           PRUint32 v = (src[0] << 24) | (src[1] << 16) | (src[2] << 8) | src[3];
           v >>= (32 - 8*aPixelSpan);
 //maybe like this:  PRUint32 v = *((PRUint32*) src) >> (32 - 8*bytesPerPix);
-#elif defined(IS_LITTLE_ENDIAN)
-          PRUint32 v = *((PRUint32*) src);
 #else
-#error Endianness not defined!
+          PRUint32 v = *((PRUint32*) src);
 #endif
 	  dest[0] = ((v & format.mRedMask) >> format.mRedShift) << (8 - format.mRedCount);
 	  dest[1] = ((v & format.mGreenMask) >> format.mGreenShift) << (8 - format.mGreenCount);
@@ -464,6 +442,93 @@ PNGWriter::ErrorCallback(png_structp mPNG,
   /* FIXME: jumpbuf stuff? */
 }
 
+/* PPM Writer */
+
+PPMWriter::PPMWriter(GtkMozEmbed *aEmbed,
+                     const char *aFilename)
+: Writer(aEmbed, aFilename)
+, mFile(NULL)
+, mRow(NULL)
+{
+  LOG ("PPMWriter ctor\n");
+}
+
+PPMWriter::~PPMWriter()
+{
+  LOG ("PPMWriter dtor\n");
+
+  if (mFile) {
+    fclose (mFile);
+  }
+  free (mRow);
+};
+
+PRBool
+PPMWriter::Prepare(nsIDrawingSurface *aSurface)
+{
+  if (mInitialised) return PR_TRUE;
+
+  mFile = fopen (mFilename.get(), "wb");
+  if (!mFile) return PR_FALSE;
+
+  fprintf(mFile, "P6 %d %d 255\n", mWidth, mHeight);
+
+  mRow = (unsigned char*) malloc(mWidth*3);
+  if (!mRow) return PR_FALSE;
+  
+  mInitialised = PR_TRUE;
+
+  return PR_TRUE;
+}
+
+void
+PPMWriter::WriteSurface(nsIDrawingSurface *aSurface,
+                        PRUint32 aWidth,
+                        PRUint32 aHeight,
+                        PRUint8 *aData,
+                        PRInt32 aRowLen,
+                        PRInt32 aRowSpan,
+                        PRInt32 aPixelSpan)
+{
+  nsPixelFormat format;
+  aSurface->GetPixelFormat(&format);
+
+  PRUint8* buf = (PRUint8*) mRow;
+  for (PRUint32 i = 0; i < aHeight; ++i)
+    {
+      PRUint8* src = aData + i*aRowSpan;
+
+      PRUint8* dest = mRow;
+      for (PRUint32 j = 0; j < aWidth; ++j)
+        {
+          /* v is the pixel value */
+#ifdef WORDS_BIGENDIAN
+          PRUint32 v = (src[0] << 24) | (src[1] << 16) | (src[2] << 8) | src[3];
+          v >>= (32 - 8*aPixelSpan);
+//maybe like this:  PRUint32 v = *((PRUint32*) src) >> (32 - 8*bytesPerPix);
+#else
+          PRUint32 v = *((PRUint32*) src);
+#endif
+          dest[0] = ((v & format.mRedMask) >> format.mRedShift) << (8 - format.mRedCount);
+          dest[1] = ((v & format.mGreenMask) >> format.mGreenShift) << (8 - format.mGreenCount);
+          dest[2] = ((v & format.mBlueMask) >> format.mBlueShift) << (8 - format.mBlueCount);
+          src += aPixelSpan;
+          dest += 3;
+        }
+
+      if (fwrite(mRow, 3, mWidth, mFile) != mWidth) {
+        mHadError = PR_TRUE;
+        break;
+      }
+    }
+}
+
+PRBool
+PPMWriter::Finish()
+{
+  return !mHadError;
+}
+
 /* GdkGdkPixData context */
 
 ThumbnailWriter::ThumbnailWriter(GtkMozEmbed *aEmbed,
@@ -528,13 +593,11 @@ ThumbnailWriter::WriteSurface(nsIDrawingSurface *aSurface,
       for (PRUint32 j = 0; j < aWidth; ++j)
         {
           /* v is the pixel value */
-#if defined(IS_BIG_ENDIAN)
+#ifdef WORDS_BIGENDIAN
           PRUint32 v = (src[0] << 24) | (src[1] << 16) | (src[2] << 8) | src[3];
           v >>= (32 - 8*aPixelSpan);
-#elif defined(IS_LITTLE_ENDIAN)
-          PRUint32 v = *((PRUint32*) src);
 #else
-#error Endianness not defined!
+          PRUint32 v = *((PRUint32*) src);
 #endif
 	  mDest[0] = ((v & format.mRedMask) >> format.mRedShift) << (8 - format.mRedCount);
 	  mDest[1] = ((v & format.mGreenMask) >> format.mGreenShift) << (8 - format.mGreenCount);
@@ -580,3 +643,123 @@ ThumbnailWriter::Finish()
 
   return retval;
 }
+
+
+#ifdef ENABLE_JPEG
+/* JPEG Writer */
+
+JPEGWriter::JPEGWriter(GtkMozEmbed *aEmbed,
+		       const char *aFilename,
+		       PRUint16 aQuality)
+: Writer(aEmbed, aFilename)
+, mQuality(aQuality)
+, mFile(NULL)
+{
+  LOG ("JPEGWriter ctor\n");
+}
+
+JPEGWriter::~JPEGWriter()
+{
+  LOG ("JPEGWriter dtor\n");
+
+  if (mFile) {
+    fclose (mFile);
+  }
+
+  jpeg_destroy_compress (&mJPEG);
+};
+
+PRBool
+JPEGWriter::Prepare(nsIDrawingSurface *aSurface)
+{
+  if (mInitialised) return PR_TRUE;
+
+  mRow = (PRUint8*) malloc (3 * mWidth);
+  if (!mRow) return PR_FALSE;
+
+  mFile = fopen (mFilename.get(), "wb");
+  if (!mFile) return PR_FALSE;
+
+  mJPEG.err = jpeg_std_error (&mHandler);
+  jpeg_create_compress (&mJPEG);
+  if (mHadError) return PR_FALSE;
+
+  jpeg_stdio_dest (&mJPEG, mFile);
+  if (mHadError) return PR_FALSE;
+
+  mJPEG.image_width = mWidth;
+  mJPEG.image_height = mHeight;
+  mJPEG.input_components = 3;
+  mJPEG.in_color_space = JCS_RGB;
+
+  jpeg_set_defaults (&mJPEG);
+  if (mHadError) return PR_FALSE;
+
+  jpeg_set_quality (&mJPEG, mQuality, TRUE /* limit to baseline-JPEG values */);
+  if (mHadError) return PR_FALSE;
+
+  jpeg_start_compress (&mJPEG, TRUE);
+  if (mHadError) return PR_FALSE;
+
+#if 0
+  nsPixelFormat format;
+  aSurface->GetPixelFormat(&format);
+  /* FIXME: assert that subsequent surfaces have same format? */
+#endif
+
+  mInitialised = PR_TRUE;
+
+  return PR_TRUE;
+}
+
+void
+JPEGWriter::WriteSurface(nsIDrawingSurface *aSurface,
+			 PRUint32 aWidth,
+			 PRUint32 aHeight,
+			 PRUint8 *aData,
+			 PRInt32 aRowLen,
+		         PRInt32 aRowSpan,
+			 PRInt32 aPixelSpan)
+{
+  nsPixelFormat format;
+  aSurface->GetPixelFormat(&format);
+
+  PRUint8* buf = (PRUint8*) mRow;
+  for (PRUint32 i = 0; i < aHeight; ++i)
+    {
+      PRUint8* src = aData + i*aRowSpan;
+
+      PRUint8* dest = buf;
+      for (PRUint32 j = 0; j < aWidth; ++j)
+        {
+          /* v is the pixel value */
+#ifdef WORDS_BIGENDIAN
+          PRUint32 v = (src[0] << 24) | (src[1] << 16) | (src[2] << 8) | src[3];
+          v >>= (32 - 8*aPixelSpan);
+//maybe like this:  PRUint32 v = *((PRUint32*) src) >> (32 - 8*bytesPerPix);
+#else
+          PRUint32 v = *((PRUint32*) src);
+#endif
+	  dest[0] = ((v & format.mRedMask) >> format.mRedShift) << (8 - format.mRedCount);
+	  dest[1] = ((v & format.mGreenMask) >> format.mGreenShift) << (8 - format.mGreenCount);
+	  dest[2] = ((v & format.mBlueMask) >> format.mBlueShift) << (8 - format.mBlueCount);
+          src += aPixelSpan;
+	  dest += 3;
+        }
+
+      JSAMPROW rows[1] = { (JSAMPLE*) mRow };
+      (void) jpeg_write_scanlines (&mJPEG, rows, 1);
+      if (mHadError) break;
+    }
+}
+
+PRBool
+JPEGWriter::Finish()
+{
+  if (mHadError) return PR_FALSE;
+
+  jpeg_finish_compress (&mJPEG);
+  return !mHadError;
+}
+
+#endif /* ENABLE_JPEG */
